unit ntkbdlites;

(***********************************************************
** NTKbdLites.pas                                         **
**                                                        **
**  Copyright 1999 Mark J. McGinty, All Rights Reserved   **
**   Free Usage granted to the public domain.             **
**   Delphi port by Tommy Sools.                          **
**                                                        **
***********************************************************)

interface

uses
  Windows, SysUtils;

const
  KEYBOARD_LED_INJECTED   = $8000; //Used by Terminal Server
  KEYBOARD_KANA_LOCK_ON   = 8; // Japanese keyboard
  KEYBOARD_CAPS_LOCK_ON   = 4;
  KEYBOARD_NUM_LOCK_ON    = 2;
  KEYBOARD_SCROLL_LOCK_ON = 1;

type
  TNtKbdLites = class
  private
    hKbdDev: THandle;
  public
    constructor Create;
    destructor Destroy; override;
    function  setLedState(indicator: UINT; state: Boolean): Boolean;
  end;

implementation

// CTL_CODE(DeviceType, Function, Method, Access) =
// (DeviceType shl 16) or (Access shl 14) or Function shl 2 or Method
const
  FILE_DEVICE_KEYBOARD = $0000000b;

  METHOD_BUFFERED   = 0;
  METHOD_IN_DIRECT  = 1;
  METHOD_OUT_DIRECT = 2;
  METHOD_NEITHER    = 3;

  FILE_ANY_ACCESS     = 0;
  FILE_SPECIAL_ACCESS = FILE_ANY_ACCESS;
  FILE_READ_ACCESS    = $0001; // file & pipe
  FILE_WRITE_ACCESS   = $0002; // file & pipe

  IOCTL_KEYBOARD_SET_INDICATORS:   DWORD = 720904;//(FILE_DEVICE_KEYBOARD shl 16) or ($0002 shl 14) or (METHOD_BUFFERED shl 2) or FILE_ANY_ACCESS;
  //IOCTL_KEYBOARD_QUERY_TYPEMATIC:  DWORD = //(FILE_DEVICE_KEYBOARD shl 16) or ($0008 shl 14) or (METHOD_BUFFERED shl 2) or FILE_ANY_ACCESS;
  IOCTL_KEYBOARD_QUERY_INDICATORS: DWORD = 720960;//(FILE_DEVICE_KEYBOARD shl 16) or ($0010 shl 14) or (METHOD_BUFFERED shl 2) or FILE_ANY_ACCESS;

type
  USHORT = Word;
  KEYBOARD_UNIT_ID_PARAMETER = record
    UnitId: USHORT;
  end;
  PKEYBOARD_UNIT_ID_PARAMETER = ^KEYBOARD_UNIT_ID_PARAMETER;
  KEYBOARD_INDICATOR_PARAMETERS = record
    // Unit identifier.  Specifies the device unit for which this
    // request is intended.
    UnitId: USHORT;
    // LED indicator state.
    LedFlags: USHORT;
  end;
  PKEYBOARD_INDICATOR_PARAMETERS = ^KEYBOARD_INDICATOR_PARAMETERS;

constructor TNtKbdLites.Create;
begin
  inherited Create;
  hKbdDev := INVALID_HANDLE_VALUE;
  if DefineDosDevice(DDD_RAW_TARGET_PATH, 'Kbd', '\Device\KeyboardClass0') then
  begin
    hKbdDev := CreateFile(
      '\\.\Kbd',
      GENERIC_WRITE,
      0,
      nil,
      OPEN_EXISTING,
      0,
      0
    );
  end;
  if hKbdDev = INVALID_HANDLE_VALUE then
    raise Exception.Create('Could not create keyboard device');
end;

destructor TNtKbdLites.Destroy;
begin
  CloseHandle(hKbdDev);
  DefineDosDevice(DDD_REMOVE_DEFINITION, 'Kbd', nil);
  inherited Destroy;
end;

function  TNtKbdLites.setLedState(indicator: UINT; state: Boolean): Boolean;
var
  bKEYBOARD_UNIT_ID_PARAMETER: KEYBOARD_UNIT_ID_PARAMETER;
  bKEYBOARD_INDICATOR_PARAMETERS: KEYBOARD_INDICATOR_PARAMETERS;
  length: DWORD;
begin
  Result := False;
  bKEYBOARD_UNIT_ID_PARAMETER.UnitId := 0;
  // read current keyboard indicator state
  if DeviceIoControl(
      hKbdDev,
      IOCTL_KEYBOARD_QUERY_INDICATORS,
      @bKEYBOARD_UNIT_ID_PARAMETER,
      SizeOf(bKEYBOARD_UNIT_ID_PARAMETER),
      @bKEYBOARD_INDICATOR_PARAMETERS,
      SizeOf(bKEYBOARD_INDICATOR_PARAMETERS),
      length,
      nil
    ) then
  begin
    // reflect target keyboard indicator state
    if state then
      // enable indicator
      bKEYBOARD_INDICATOR_PARAMETERS.LedFlags := bKEYBOARD_INDICATOR_PARAMETERS.LedFlags or indicator
    else
      // disable indicator
      bKEYBOARD_INDICATOR_PARAMETERS.LedFlags := bKEYBOARD_INDICATOR_PARAMETERS.LedFlags and not(indicator);
    // set new keyboard indicator state
    if DeviceIoControl(
        hKbdDev,
        IOCTL_KEYBOARD_SET_INDICATORS,
        @bKEYBOARD_INDICATOR_PARAMETERS,
        SizeOf(bKEYBOARD_INDICATOR_PARAMETERS),
        nil,
        0,
        length,
        nil
      ) then
      Result := True;
  end;
end;

end.
