//--------------------------------------------------------------------------
//     SmallPort ver 1.4 for Delphi 4 modified 21-August-2000
//     Windows NT/2000 compatible
//     Author: A.Weitzman
//     Modified by Tommy Sools:
//     simplified and clean up in order to reduce file size.
//     modified code to make it more error proof.
//--------------------------------------------------------------------------

unit usmallport;

interface

uses
  Windows, SysUtils, Classes, Winsvc;

type
  TBytePort = record address: WORD; value: BYTE; end;
  TSmallPort = class(TComponent)
  private
   dwLastError: DWORD;
   dwReadByteCode:   LongInt;
   dwWriteByteCode:  LongInt;
   dwReadWordCode:   LongInt;
   dwWriteWordCode:  LongInt;
   dwReadDWordCode:  LongInt;
   dwWriteDWordCode: LongInt;
   DriverName: string;
  protected
    hDevice: SC_HANDLE;
    cbBytesReturned: DWORD;
    ByteData: TBytePort;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy(); override;
    function OpenDriver(): Boolean;
    procedure CloseDriver();
    function IsOpen(): Boolean;
    function ReadByte(Index: WORD): Byte;
    procedure WriteByte(Index: WORD; value: Byte);
    function ServiceExists(): Boolean;
    function ServiceStatus(): DWORD;
    function ServicePath(): string;
    procedure ServiceInstall(autoStart: Boolean);
    procedure ServiceRemove();
    procedure ServiceStart();
    procedure ServiceStop();
    property Port[Index1:WORD]: Byte read ReadByte write WriteByte;
    property LastError: DWORD read dwLastError;
  published
    property DriverHandle: DWORD read hDevice default 0;
  end;

procedure Register;

const
  SMALLPORT_TYPE      = 61696;

  METHOD_BUFFERED     = 0;
  FILE_READ_ACCESS    = 1;
  FILE_WRITE_ACCESS   = 2;

  IOCTL_NT_READBYTE = (SMALLPORT_TYPE shl 16) or (FILE_READ_ACCESS shl 14) or ($901 shl 2) or METHOD_BUFFERED;
  IOCTL_NT_WRITEBYTE = (SMALLPORT_TYPE shl 16) or (FILE_WRITE_ACCESS shl 14) or ($902 shl 2) or METHOD_BUFFERED;
  IOCTL_NT_READWORD = (SMALLPORT_TYPE shl 16) or (FILE_READ_ACCESS shl 14) or ($903 shl 2) or METHOD_BUFFERED;
  IOCTL_NT_WRITEWORD = (SMALLPORT_TYPE shl 16) or (FILE_WRITE_ACCESS shl 14) or ($904 shl 2) or METHOD_BUFFERED;
  IOCTL_NT_READDWORD = (SMALLPORT_TYPE shl 16) or (FILE_READ_ACCESS shl 14) or ($905 shl 2) or METHOD_BUFFERED;
  IOCTL_NT_WRITEDWORD = (SMALLPORT_TYPE shl 16) or (FILE_WRITE_ACCESS shl 14) or ($906 shl 2) or METHOD_BUFFERED;

  IOCTL_READBYTE = 2;
  IOCTL_WRITEBYTE = 3;
  IOCTL_READWORD = 4;
  IOCTL_WRITEWORD = 5;
  IOCTL_READDWORD = 6;
  IOCTL_WRITEDWORD = 7;

implementation

procedure Register;
begin
  RegisterComponents('System', [TSmallPort]);
end;

function CTL_CODE(DeviceType: integer; func: integer; meth: integer; access: integer): DWORD;
Begin
  result := (DeviceType shl 16) or (Access shl 14) or (func shl 2) or (meth);
end;

function WindowsVersionNT(): Boolean;
begin
  Result := GetVersion() < $80000000;
end;

constructor TSmallPort.Create(aOwner: TComponent);
begin
 inherited Create(aOwner);
 hDevice := INVALID_HANDLE_VALUE;
 DriverName := 'Smport';
 if WindowsVersionNT() then
 begin
   dwReadByteCode   := IOCTL_NT_READBYTE;
   dwWriteByteCode  := IOCTL_NT_WRITEBYTE;
   dwReadWordCode   := IOCTL_NT_READWORD;
   dwWriteWordCode  := IOCTL_NT_WRITEWORD;
   dwReadDWordCode  := IOCTL_NT_READDWORD;
   dwWriteDWordCode := IOCTL_NT_WRITEDWORD;
 end
 else
 begin
   dwReadByteCode   := IOCTL_READBYTE;
   dwWriteByteCode  := IOCTL_WRITEBYTE;
   dwReadWordCode   := IOCTL_READWORD;
   dwWriteWordCode  := IOCTL_WRITEWORD;
   dwReadDWordCode  := IOCTL_READDWORD;
   dwWriteDWordCode := IOCTL_WRITEDWORD;
 end;
 dwLastError := 0;
end;

destructor TSmallPort.Destroy();
begin
  if (hDevice <> INVALID_HANDLE_VALUE) then
    CloseDriver();
  inherited Destroy();
end;

function TSmallPort.OpenDriver(): Boolean;
var
  buf: PChar;
  s: string;
  status: DWORD;
begin
 Result := false;
 if hDevice <> INVALID_HANDLE_VALUE then
 begin
   CloseDriver();
   hDevice := INVALID_HANDLE_VALUE;
 end;
 if WindowsVersionNT() then
 begin
   // check if driver already exists
   if ServiceExists() then
   begin
     // check if driver is already started
     status := ServiceStatus();
     if (status = SERVICE_STOPPED) or (status = SERVICE_PAUSED) then
     begin
       // driver is not started yet, attempt to start driver
       ServiceStart();
     end;
     // attempt to start communicating with driver
     hDevice :=
       CreateFile(
         PChar('\\.\'+ DriverName),
         GENERIC_READ OR GENERIC_WRITE,
         0,
         PSECURITY_DESCRIPTOR(nil),
         OPEN_EXISTING,
         FILE_ATTRIBUTE_NORMAL,
         0
       );
   end;
 end
 else
 begin
   // try to open VXD driver from current directory
   hDevice := CreateFile( '\\.\smport.vxd', 0, 0, nil, 0, 0, 0);
   if hDevice = INVALID_HANDLE_VALUE then
   begin
     // try to open VXD driver from system directory
     GetMem(buf, MAX_PATH + 1);
     try
       GetSystemDirectory(buf, MAX_PATH);
       s := '\\.\' + buf +'\smport.vxd';
     finally
       FreeMem(buf);
     end;
     hDevice := CreateFile(PChar(s), 0, 0, nil, 0, 0, 0);
     if hDevice = INVALID_HANDLE_VALUE then
       dwLastError := GetLastError();
   end;
 end;
 if hDevice <> INVALID_HANDLE_VALUE then
   Result := True;
end;

procedure TSmallPort.CloseDriver();
begin
  if hDevice <> INVALID_HANDLE_VALUE then
  begin
    CloseHandle(hDevice);
    hDevice := INVALID_HANDLE_VALUE;
  end;
end;

function TSmallPort.IsOpen(): boolean;
begin
  Result := hDevice <> INVALID_HANDLE_VALUE;
end;

function TSmallPort.ReadByte(Index: WORD): Byte;
var
  value: Word;
begin
 if hDevice = INVALID_HANDLE_VALUE then
 begin
   Result := 0;
   dwLastError := ERROR_INVALID_HANDLE;
 end
 else
 begin
   if DeviceIoControl(
     hDevice,
     dwReadByteCode,
     @Index,
     sizeof(Index),
     @value,
     sizeof(value),
     cbBytesReturned,
     nil
   ) then
   begin
     Result := Lo(value);
   end
   else
   begin
     dwLastError := GetLastError;
     Result := 0;
   end;
  end;
end;

procedure TSmallPort.WriteByte(Index: WORD; value: Byte);
begin
 if hDevice = INVALID_HANDLE_VALUE then
 begin
   dwLastError := ERROR_INVALID_HANDLE;
 end
 else
 begin
   ByteData.address := Index;
   ByteData.value := value;
   if not(
     DeviceIoControl(
       hDevice,
       dwWriteByteCode,
       @ByteData,
       sizeof(ByteData),
       @value,
       sizeof(value),
       cbBytesReturned,
       nil
     )) then
   begin
     dwLastError := GetLastError;
   end;
 end;
end;

function TSmallPort.ServiceExists(): Boolean;
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
begin
  Result := False;
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), SERVICE_QUERY_STATUS);
    if (hService <> 0) then
    begin
      CloseServiceHandle(hService);
      Result := True;
    end
    else
      Result := False;
    CloseServiceHandle(hManager);
  end;
end;

function TSmallPort.ServiceStatus(): DWORD;
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  serviceStatus: SERVICE_STATUS;
begin
  Result := SERVICE_STOPPED;
  dwLastError := 0;
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), SERVICE_QUERY_STATUS);
    if hService <> 0 then
    begin
      if QueryServiceStatus(hService, serviceStatus) then
        Result := serviceStatus.dwCurrentState
      else
        dwLastError := GetLastError;
      CloseServiceHandle(hService);
    end
    else
      dwLastError := GetLastError;
    CloseServiceHandle(hManager);
  end
  else
    dwLastError := GetLastError();
end;

function TSmallPort.ServicePath(): string;
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  serviceConfig: PQueryServiceConfig;
  length, returnLength: DWORD;
begin
  Result := '';
  dwLastError := 0;
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), SERVICE_QUERY_CONFIG);
    if hService <> 0 then
    begin
      QueryServiceConfig(hService, nil, 0, length);
      GetMem(serviceConfig, length);
      try
        if QueryServiceConfig(hService, serviceConfig, length, returnLength) then
          Result := serviceConfig^.lpBinaryPathName
        else
          dwLastError := GetLastError();
      finally
        FreeMem(serviceConfig);
      end;
      CloseServiceHandle(hService);
    end
    else
      dwLastError := GetLastError();
    CloseServiceHandle(hManager);
  end
  else
    dwLastError := GetLastError();
end;

procedure TSmallPort.ServiceInstall(autoStart: Boolean);
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  SourcePath: string;
  DriverPath: string;
  startType: DWORD;
  buf: PChar;
begin
  dwLastError := 0;
  if ServiceExists() then
    ServiceRemove();
  GetMem(buf, MAX_PATH + 1);
  try
    if GetSystemDirectory(buf, MAX_PATH) > 0 then
      DriverPath := buf + '\drivers\smport.sys';
  finally
    FreeMem(buf);
  end;
  SourcePath := ExtractFilePath(ParamStr(0)) + 'smport.sys';
  CopyFile(
    PChar(SourcePath),
    PChar(DriverPath),
    False
  );
  if FileExists(DriverPath) then
  begin
    hManager := OpenSCManager(nil, nil, SC_MANAGER_CREATE_SERVICE);
    if hManager <> 0 then
    begin
      if autoStart then
        startType := SERVICE_BOOT_START
      else
        startType := SERVICE_DEMAND_START;
      hService :=
        CreateService(
          hManager,
          PChar(DriverName),
          PChar(DriverName),
          SERVICE_QUERY_STATUS,
          SERVICE_KERNEL_DRIVER,
          startType,
          SERVICE_ERROR_NORMAL,
          PChar(DriverPath),
          nil, nil, nil, nil, nil
        );
      if hService <> 0 then
        CloseServiceHandle(hService)
      else
        dwLastError := GetLastError;
      CloseServiceHandle(hManager);
      if autoStart then
        ServiceStart();
    end
    else
      dwLastError := GetLastError;
  end;
end;

procedure TSmallPort.ServiceRemove();
var
  path: string;
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
begin
  dwLastError := 0;
  ServiceStop();
  path := ServicePath();
  if dwLastError <> 0 then
    path := '';
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), _DELETE);
    if hService <> 0 then
    begin
      if not(DeleteService(hService)) then
        dwLastError := GetLastError();
      CloseServiceHandle(hService);
    end
    else
      dwLastError := GetLastError();
    CloseServiceHandle(hManager);
    if FileExists(path) then
      DeleteFile(path);
  end
  else
    dwLastError := GetLastError();
end;

procedure TSmallPort.ServiceStart();
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  lpVectors: PChar;
begin
  dwLastError := 0;
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), SERVICE_START);
    if hService <> 0 then
    begin
      lpVectors := nil;
      if not(StartService(hService, 0, lpVectors)) then
        dwLastError := GetLastError;
      CloseServiceHandle(hService);
    end
    else
      dwLastError := GetLastError();
    CloseServiceHandle(hManager);
  end
  else
    dwLastError := GetLastError();
end;

procedure TSmallPort.ServiceStop();
var
  hManager: SC_HANDLE;
  hService: SC_HANDLE;
  Status: TServiceStatus;
begin
  dwLastError := 0;
  hManager := OpenSCManager(nil, nil, SC_MANAGER_CONNECT);
  if hManager <> 0 then
  begin
    hService := OpenService(hManager, PChar(DriverName), SERVICE_STOP);
    if hService <> 0 then
    begin
      if not(ControlService(hService, SERVICE_CONTROL_STOP, Status)) then
        dwLastError := GetLastError;
      CloseServiceHandle(hService);
    end
    else
      dwLastError := GetLastError();
    CloseServiceHandle(hManager);
  end
  else
    dwLastError := GetLastError();
end;

end.
