unit unitformconfiguration;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Lightning configuration dialog.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

 interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls;

type
  TformConfiguration = class(TForm)
    pageControlTabs: TPageControl;
    tabSheetHardware: TTabSheet;
    groupBoxStartup: TGroupBox;
    buttonOK: TButton;
    buttonCancel: TButton;
    checkBoxLaunchDMXLink: TCheckBox;
    tabSheetTimers: TTabSheet;
    groupBoxTimerProcessing: TGroupBox;
    labelWarning: TLabel;
    groupBoxSubTimers: TGroupBox;
    labelModules: TLabel;
    labelModulesHz: TLabel;
    listBoxTabs: TListBox;
    checkBoxLaunchBDetect: TCheckBox;
    tabSheetSmallport: TTabSheet;
    groupBoxSmallport: TGroupBox;
    buttonSmallportInstallAuto: TButton;
    buttonSmallportRemove: TButton;
    labelSmallportLocation: TLabel;
    labelSmallportLocation2: TLabel;
    panelTimers: TPanel;
    panelStartup: TPanel;
    panelSmallport: TPanel;
    buttonSmallportInstallManual: TButton;
    buttonSmallportStop: TButton;
    labelSmallportStarted: TLabel;
    labelSmallportStopped: TLabel;
    buttonSmallportStart: TButton;
    tabSheetAppearance: TTabSheet;
    groupBoxAppearance: TGroupBox;
    checkBoxStartMinimized: TCheckBox;
    checkBoxSystemTray: TCheckBox;
    panelAppearance: TPanel;
    checkBoxSplash: TCheckBox;
    checkBoxAlwaysOnTop: TCheckBox;
    checkBoxTooltips: TCheckBox;
    groupBoxShutdown: TGroupBox;
    checkBoxCloseDMXLink: TCheckBox;
    checkBoxCloseBDetect: TCheckBox;
    labelInformation: TLabel;
    labelDefault: TLabel;
    groupBoxTimerGraphics: TGroupBox;
    trackBarTimerGraphics: TTrackBar;
    labelTimerGraphicsFast: TLabel;
    labelTimerGraphicsSlow: TLabel;
    labelTimerProcessingSlow: TLabel;
    trackBarTimerProcessing: TTrackBar;
    labelTimerProcessingFast: TLabel;
    labelGraphics: TLabel;
    labelGraphicsHz: TLabel;
    labelProcessingHz: TLabel;
    labelProcessing: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure listBoxTabsClick(Sender: TObject);
    procedure tabSheetTimersShow(Sender: TObject);
    procedure tabSheetSmallportShow(Sender: TObject);
    procedure buttonSmallportInstallAutoClick(Sender: TObject);
    procedure buttonSmallportInstallManualClick(Sender: TObject);
    procedure buttonSmallportRemoveClick(Sender: TObject);
    procedure buttonSmallportStartClick(Sender: TObject);
    procedure buttonSmallportStopClick(Sender: TObject);
    procedure labelDefaultClick(Sender: TObject);
    procedure trackBarTimerProcessingChange(Sender: TObject);
    procedure trackBarTimerGraphicsChange(Sender: TObject);
  private
    { Private declarations }
    procedure executeSmport(param: string);
  public
    { Public declarations }
    procedure setTimerProcessing(interval: Integer);
    function getTimerProcessing(): Integer;
    procedure setTimerGraphics(interval: Integer);
    function getTimerGraphics(): Integer;
  end;

var
  formConfiguration: TformConfiguration;

implementation

{$R *.dfm}

uses
  common, unitlightningsettings, usmallport, Winsvc, Shellapi;

const
  BCM_SETSHIELD = $0000160C;

procedure TformConfiguration.FormCreate(Sender: TObject);
begin
  pageControlTabs.ActivePageIndex := 0;
  SendMessage(buttonSmallportInstallAuto.Handle, BCM_SETSHIELD, 0, 1);
  SendMessage(buttonSmallportInstallManual.Handle, BCM_SETSHIELD, 0, 1);
  SendMessage(buttonSmallportRemove.Handle, BCM_SETSHIELD, 0, 1);
  SendMessage(buttonSmallportStart.Handle, BCM_SETSHIELD, 0, 1);
  SendMessage(buttonSmallportStop.Handle, BCM_SETSHIELD, 0, 1);
end;

procedure TformConfiguration.FormShow(Sender: TObject);
begin
  listBoxTabsClick(Sender);
  listBoxTabs.SetFocus();
end;

procedure TformConfiguration.listBoxTabsClick(Sender: TObject);
begin
  if listBoxTabs.ItemIndex = -1 then
    listBoxTabs.ItemIndex := pageControlTabs.ActivePageIndex;
  pageControlTabs.ActivePageIndex := listBoxTabs.ItemIndex;
  if pageControlTabs.ActivePage = tabSheetSmallport then
    tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.tabSheetTimersShow(Sender: TObject);
begin
  if tickFreq <> 0 then
    labelModulesHz.Caption := IntToStr(tickFreq) + ' Hz'
  else
    labelModulesHz.Caption := '< 1000 Hz';
end;

procedure TformConfiguration.tabSheetSmallportShow(Sender: TObject);
var
  bSmallPort: TSmallPort;
  status: DWORD;
  started: Boolean;
begin
  bSmallPort := TSmallPort.Create(nil);
  try
    labelSmallportLocation2.Caption := bSmallPort.ServicePath();
    if bSmallPort.LastError = ERROR_SERVICE_DOES_NOT_EXIST then
      labelSmallportLocation2.Caption := '(smport.sys not found)'
    else if bSmallPort.LastError <> 0 then
      labelSmallportLocation2.Caption := '(error ' + IntToStr(bSmallPort.LastError) + ')';
    if bSmallPort.LastError = 0 then
    begin
      status := bSmallPort.ServiceStatus();
      started := (status <> SERVICE_STOPPED) and (status <> SERVICE_PAUSED);
      labelSmallportStarted.Visible := started;
      labelSmallportStopped.Visible := not(started);
      buttonSmallportInstallAuto.Caption := 'R&epair driver (auto start)';
      buttonSmallportInstallManual.Caption := 'R&epair driver (manual start)';
      buttonSmallportRemove.Enabled := True;
      buttonSmallportStart.Enabled := not(started);
      buttonSmallportStop.Enabled := started;
    end
    else
    begin
      labelSmallportStarted.Hide();
      labelSmallportStopped.Hide();
      buttonSmallportInstallAuto.Caption := '&Install driver (auto start)';
      buttonSmallportInstallManual.Caption := '&Install driver (manual start)';
      buttonSmallportRemove.Enabled := False;
      buttonSmallportStart.Enabled := False;
      buttonSmallportStop.Enabled := False;
    end;
  finally
    bSmallPort.Free();
  end;
end;

procedure TformConfiguration.executeSmport(param: string);
var
  path, fileName: string;
  bShellExecuteInfo: SHELLEXECUTEINFO;
begin
  // reset SmallPort query dialog memory
  lightningSettings.smallPortQuery := False;
  // smport.exe requires administrator privileges
  path := ExtractFilePath(ParamStr(0));
  fileName := path + 'smport.exe';
  with bShellExecuteInfo do
  begin
    cbSize := sizeof(bShellExecuteInfo);
    fMask := SEE_MASK_NOCLOSEPROCESS;
    Wnd := Self.Handle;
    lpVerb := nil;
    lpFile := PChar(fileName);
    lpParameters := PChar(param);
    lpDirectory := PChar(path);
    nShow := SW_HIDE;
  end;
  ShellExecuteEx(@bShellExecuteInfo);
  WaitForSingleObject(bShellExecuteInfo.hProcess, 5000); // 5 seconds timeout
  CloseHandle(bShellExecuteInfo.hProcess);
end;

procedure TformConfiguration.buttonSmallportInstallAutoClick(Sender: TObject);
begin
  executeSmport('installauto');
  tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.buttonSmallportInstallManualClick(
  Sender: TObject);
begin
  executeSmport('installmanual');
  tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.buttonSmallportRemoveClick(Sender: TObject);
begin
  executeSmport('remove');
  tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.buttonSmallportStartClick(Sender: TObject);
begin
  executeSmport('start');
  tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.buttonSmallportStopClick(Sender: TObject);
begin
  executeSmport('stop');
  tabSheetSmallportShow(Sender);
end;

procedure TformConfiguration.labelDefaultClick(Sender: TObject);
begin
  setTimerProcessing(10);
end;

procedure TformConfiguration.trackBarTimerProcessingChange(Sender: TObject);
begin
  labelWarning.Visible      := getTimerProcessing() <> 10;
  labelDefault.Visible      := getTimerProcessing() <> 10;
  labelInformation.Visible  := getTimerProcessing() = 10;
  labelProcessingHz.Caption := IntToStr(1000 div getTimerProcessing()) + ' Hz'
end;

procedure TformConfiguration.setTimerProcessing(interval: Integer);
begin
  trackBarTimerProcessing.Position := 1 + trackBarTimerProcessing.Max - interval;
  trackBarTimerProcessingChange(nil);
end;

function TformConfiguration.getTimerProcessing(): Integer;
begin
  Result := 1 + trackBarTimerProcessing.Max - trackBarTimerProcessing.Position;
end;

procedure TformConfiguration.setTimerGraphics(interval: Integer);
begin
  trackBarTimerGraphics.Position := 1 + trackBarTimerGraphics.Max - interval;
  trackBarTimerGraphicsChange(nil);
end;

function TformConfiguration.getTimerGraphics(): Integer;
begin
  Result := 1 + trackBarTimerGraphics.Max - trackBarTimerGraphics.Position;
end;

procedure TformConfiguration.trackBarTimerGraphicsChange(Sender: TObject);
begin
  labelGraphicsHz.Caption := IntToStr(1000 div getTimerGraphics()) + ' Hz';
  labelGraphics.Visible   := getTimerGraphics() > 1;
  labelGraphicsHz.Visible := getTimerGraphics() > 1;
end;

end.
