unit unitformscenemanager;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Lightning scene manager window.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Menus, ImgList, ComCtrls, ToolWin,
  common, unitformlightning;

type
  TScene = record
    name: string;
    fadeTime, advanceTime, advanceTimeCountDown: Integer;
    itemIndex, itemData: TStringList;
  end;
  PScene = ^TScene;
  TformSceneManager = class(TForm)
    lbScenes: TListBox;
    lFadeTime: TLabel;
    lFadeTimeNotice: TLabel;
    chNoSceneSwitch: TCheckBox;
    chOverride: TCheckBox;
    pmFadeTime: TPopupMenu;
    miApplytoAllScenesinCurrentProgram: TMenuItem;
    tbSceneManager: TToolBar;
    tbNew: TToolButton;
    tbRecall: TToolButton;
    tbRename: TToolButton;
    tbDuplicate: TToolButton;
    tbDelete: TToolButton;
    tbStore: TToolButton;
    tbProperties: TToolButton;
    tbSep1: TToolButton;
    tbUp: TToolButton;
    tbDown: TToolButton;
    tbSep2: TToolButton;
    editFadeTime: TEdit;
    upDownFadeTime: TUpDown;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cleanup;
    procedure loadProgram(fileName: string);
    procedure saveProgram(fileName, programName: string);
    procedure miApplyToAllScenesinCurrentProgramClick(Sender: TObject);
    procedure tbNewClick(Sender: TObject);
    procedure tbRecallClick(Sender: TObject);
    procedure tbStoreClick(Sender: TObject);
    procedure tbRenameClick(Sender: TObject);
    procedure tbDeleteClick(Sender: TObject);
    procedure tbPropertiesClick(Sender: TObject);
    procedure tbUpClick(Sender: TObject);
    procedure tbDownClick(Sender: TObject);
  private
    { Private declarations }
    currentScene:  PScene;
    lastFadeStart: Int64;   // modified 2004-11-10
    lastFadeTime:  Integer; // modified 2004-11-10
    procedure storeScene(index: Integer);
  public
    { Public declarations }
    procedure nextScene();
    procedure recallScene(ignoreFadeTimeOnFile: Boolean);
    function decrementAdvanceTime: Boolean;
  end;

var
  formSceneManager: TformSceneManager;
  bProgram: TProgram = nil;
  lastTime: Int64 = 0;

implementation

{$R *.dfm}

uses
  unitlightningsettings, unitmodule, unitformprogrammanager, unitformsceneproperties;

procedure TformSceneManager.FormCreate(Sender: TObject);
begin
  currentScene := nil;
  bProgram := TProgram.Create;
  tickInit;
  lastTime := tickCount;
  lastFadeStart := tickCount; // modified 2004-11-10
  lastFadeTime  := 0; // modified 2004-11-10
end;

procedure TformSceneManager.FormDestroy(Sender: TObject);
begin
  bProgram.Free;
end;

procedure TformSceneManager.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caNone;
  formLightning.miSceneManagerClick(Sender);
end;

(*procedure TfSceneManager.bDuplicateClick(Sender: TObject);
var
  s: Integer;
  bPScene, bPScene2: PScene;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then // check if any scene is actually selected
  begin
    // store reference to source scene
    bPScene2 := PScene(bProgram.Items[s]);

    // create new scene
    New(bPScene);
    with bPScene^ do
    begin
      name                 := bPScene2^.name;
      fadeTime             := bPScene2^.fadeTime;
      advanceTime          := bPScene2^.advanceTime;
      advanceTimeCountDown := bPScene2^.advanceTimeCountDown;
      itemIndex            := TStringList.Create;
      itemIndex.Assign(bPScene2^.itemIndex);
      itemData             := TStringList.Create;
      itemData.Assign(bPScene2^.itemData);
      bProgram.Add(bPScene);
      lbScenes.Items.Add(name);
    end;
  end;
end;*)

procedure TformSceneManager.storeScene(index: Integer);
{
  Pre: index is a valid index of a scene in the current program;
  Post: the scene pointed to by index in the current program has been updated with the current desktop settings;
}
var
  bModule:  TModule;
  bPScene:  PScene;
  settings: string;
  i:        Integer;
begin
  bPScene := PScene(bProgram.Items[index]);
  bPScene^.fadeTime := upDownFadeTime.Position;
  bPScene^.itemIndex.Clear;
  bPScene^.itemData.Clear;
  for i := 0 to Length(formLightning.dvDesktop.modules) - 1 do
  begin
    bModule := formLightning.dvDesktop.modules[i];
    if (bModule.sceneid <> '') and (@bModule.openscene <> nil) and bModule.usedForScene then
    begin
      bPScene^.itemIndex.Add(bModule.sceneid);
      SetLength(settings, maxLength);
      SetLength(settings, bModule.savescene(bModule.module, PChar(settings), Length(settings)));
      bPScene^.itemData.Add(settings);
    end;
  end;
end;

procedure TformSceneManager.nextScene();
begin
  if not(chNoSceneSwitch.Checked) then
  begin
    if lbScenes.Count > 0 then
    begin
      // if a fade time was set,
      // make sure to only load the next scene if the fade has finished
      // if the corresponding option is set
      if not(lightningSettings.alwaysFinishFades)
         or
         (lastFadeTime <= 0)
         or
         ((lastFadeTime > 0) and (tickCount - lastFadeStart > lastFadeTime)) then
      begin
        if lbScenes.ItemIndex = lbScenes.Count - 1 then
          lbScenes.ItemIndex := 0
        else
          lbScenes.ItemIndex := lbScenes.ItemIndex + 1;
        recallScene(False);
      end;
    end;
  end;
end;

procedure TformSceneManager.recallScene(ignoreFadeTimeOnFile: Boolean);
var
  bModule:  TModule;
  bPScene:  PScene;
  settings: string;
  s, i, j, currentFadeTime, fadeTime: Integer;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then
  begin
    bPScene := PScene(bProgram[s]);
    currentScene := bPScene; // modified 2004-02-26
    if not(ignoreFadeTimeOnFile) and not(chOverride.Checked) then
      if useCustomFadeTime <> -1 then
        upDownFadeTime.Position := useCustomFadeTime
      else
        upDownFadeTime.Position := bPScene^.fadeTime;
    currentFadeTime := upDownFadeTime.Position;
    lastFadeTime  := currentFadeTime; // modified 2004-11-10
    lastFadeStart := tickCount; // modified 2004-11-10
    bPScene^.advanceTimeCountDown := MaxInt; // modified 2004-02-26
    for i := 0 to Length(formLightning.dvDesktop.modules) - 1 do
    begin
      bModule := formLightning.dvDesktop.modules[i];
      if bModule.sceneid <> '' then
      begin
        j := bPScene^.itemIndex.IndexOf(bModule.sceneid);
        if @bModule.openscene <> nil then // check if this module is scene capable
          if j > -1 then // check if module is enabled for this scene
          begin
            // module is enabled for this scene
            settings := bPScene^.itemData[j];
            if bModule.excludeFromFadeTime then
              fadeTime := 0
            else
              fadeTime := {fProgramManager}currentFadeTime{fSceneManager.upDownFadeTime.Position}{fadeTime{2000{20000{0}; // modified 2004-11-10
            bModule.setIsUsedForScene(True);
            bModule.openscene(bModule.module, PChar(settings), fadeTime, nil);
          end
          else
            // module is not enabled for this scene
            bModule.setIsUsedForScene(False);
      end;
    end;
  end;
end;

procedure TformSceneManager.cleanup;
var
  i: Integer;
begin
  formLightning.stopTimer();
  try
    currentScene := nil;
    lbScenes.Clear;
    for i := 0 to bProgram.Count - 1 do
    begin
      PScene(bProgram[i])^.itemData.Free;
      PScene(bProgram[i])^.itemIndex.Free;
    end;
    bProgram.Clear;
  finally
    formLightning.startTimer();
  end;
end;

procedure TformSceneManager.loadProgram(fileName: string);
var
  bStringList: TStringList;
  bPScene:     PScene;
  item:        string;
  i:           Integer;
  bModule:     TModule;
begin
  formLightning.stopTimer();
  try
    cleanup();
    bStringList := TStringList.Create;
    try
      bStringList.LoadFromFile(fileName);
      if bStringList.Count > 0 then
      begin
        item := bStringList[0];
        currentProgramName := parse(item, '|');
        if not(formProgramManager.chOverride.Checked) then
          formProgramManager.upDownFadeTime.Position := StrToIntDef(parse(item, '|'), 0);
        if not(chOverride.Checked) then
          upDownFadeTime.Position := formProgramManager.upDownFadeTime.Position;
        bStringList.Delete(0);
        while bStringList.Count > 0 do
        begin
          New(bPScene);
          bProgram.Add(bPScene);
          if bStringList.Count >= 2 then
          begin
            item := bStringList[0];
            with bPScene^ do
            begin
              name     := parse(item, '|');
              fadeTime := StrToIntDef(parse(item, '|'), 0);
              advanceTime := StrToIntDef(parse(item, '|'), -1);
              advanceTimeCountDown := MaxInt;
            end;
            lbScenes.Items.Add(bPScene^.name);
            bStringList.Delete(0);
            bPScene^.itemIndex := TStringList.Create;
            bPScene^.itemData  := TStringList.Create;
            while (bStringList.Count > 0) and (bStringList[0] <> '') do
            begin
              item := bStringList[0];
              bStringList.Delete(0);
              bPScene^.itemIndex.Add(parse(item, '|'));
              bPScene^.itemData.Add(item);
            end;
            if bStringList.Count > 0 then
              bStringList.Delete(0);
          end;
        end;
      end
      else
        ShowMessage('Program file is empty!');
    finally
      bStringList.Free;
    end;
    lbScenes.ItemIndex := -1;
    if lbScenes.Count > 0 then
    begin
      // at least one scene exists in the program,
      // load the first scene.
      lbScenes.ItemIndex := 0;
      recallScene(True);
    end
    else
    begin
      // no scenes exist in the program,
      // disable all "used for scene" buttons on all scene capable modules.
      for i := 0 to Length(formLightning.dvDesktop.modules) - 1 do
      begin
        bModule := formLightning.dvDesktop.modules[i];
        if @bModule.openscene <> nil then
        begin
          // if this module is scene capable,
          // so disable its "used for scene" button.
          bModule.setIsUsedForScene(False);
        end;
      end;
    end;
  finally
    formLightning.startTimer();
  end;
end;

procedure TformSceneManager.saveProgram(fileName, programName: string);
var
  bStringList: TStringList;
  bPScene:     PScene;
  i, j:        Integer;
begin
  bStringList := TStringList.Create;
  try
    bStringList.Add(programName + '|' + IntToStr(formProgramManager.upDownFadeTime.Position));
    for i := 0 to bProgram.Count - 1 do
    begin
      bPScene := PScene(bProgram[i]);
      bStringList.Add(bPScene.name{'Scene|'} + '|' + IntToStr(bPScene.fadeTime) + '|' + IntToStr(bPScene.advanceTime)); // scene name
      for j := 0 to bPScene^.itemIndex.Count - 1 do
        bStringList.Add(bPScene^.itemIndex[j] + '|' + bPScene^.itemData[j]);
      bStringList.Add('');
    end;
    bStringList.SaveToFile(fileName);
  finally
    bStringList.Free();
  end;
end;

procedure TformSceneManager.miApplyToAllScenesinCurrentProgramClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to bProgram.Count - 1 do
    PScene(bProgram[i])^.fadeTime := upDownFadeTime.Position;
end;

function TformSceneManager.decrementAdvanceTime: Boolean;
var
  //s:       Integer;
  bPScene: PScene;
  time:    Int64;
begin
  Result := False;
  if currentScene <> nil then
  begin
    bPScene := currentScene;
    if (bPScene^.advanceTime > -1) and (bPScene^.advanceTimeCountDown > -1) then
    begin
      time := tickCount;
      if bPScene^.advanceTimeCountDown = MaxInt then
        bPScene^.advanceTimeCountDown := bPScene^.advanceTime
      else
      begin
        Dec(bPScene^.advanceTimeCountDown, time - lastTime);
        Result := bPScene^.advanceTimeCountDown < 1;
      end;
      lastTime := time;
    end;
  end;
end;

procedure TformSceneManager.tbNewClick(Sender: TObject);
{
  Pre:  a program was loaded;
  Post: if the loaded program contains no scenes yet, then an empty scene with all "Used for scene" disabled was created or
        if the loaded program already contains any scenes, a new scene will be added according to the current desktop settings;
}
var
  bPScene: PScene;
//  bModule:  TModule;
//  settings: string;
//  i:        Integer;
begin
  New(bPScene);
  with bPScene^ do
  begin
    name                 := 'Scene';
    fadeTime             := upDownFadeTime.Position;
    advanceTime          := -1;
    advanceTimeCountDown := MaxInt;
    itemIndex            := TStringList.Create;
    itemData             := TStringList.Create;
    bProgram.Add(bPScene);
    lbScenes.Items.Add(name);
  end;

(*  if bProgram.Count = 1 then
  begin
    // this is the first scene in the program, so create an empty new scene
  end
  else
  begin
    // this isn't the first scene in the program, so use the current desktop settings for the scene
*)
    storeScene(bProgram.Count - 1);
    {
    // create a new scene according to the last scene in the program
    bPScene2 := PScene(bProgram.Items[bProgram.Count - 2]);
    bPScene^.itemIndex.Assign(bPScene2^.itemIndex);
    bPScene^.itemData.Assign(bPScene2^.itemData);}
    {for i := 0 to Length(modules) - 1 do
    begin
      bModule := modules[i];
      if (bModule.sceneid <> '') and (@bModule.openscene <> nil) and bModule.usedForScene then
      begin
        bPScene^.itemIndex.Add(bModule.sceneid);

        SetLength(settings, maxLength);
        SetLength(settings, bModule.savescene(bModule.module, PChar(settings), Length(settings)));

        bPScene^.itemData.Add(settings);
      end;
    end;}
(*  end;*)

  // recall the last scene in the current program, which is the one that has just been created
  lbScenes.ItemIndex := lbScenes.Count - 1;
  recallscene(False);
end;

procedure TformSceneManager.tbRecallClick(Sender: TObject);
begin
  recallScene(False);
end;

procedure TformSceneManager.tbStoreClick(Sender: TObject);
{
  Pre: true;
  Post: if an existing scene was selected, the current settings were stored in the selected scene;
}
var
  s: Integer;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then
    storeScene(s);
end;

procedure TformSceneManager.tbRenameClick(Sender: TObject);
var
  s:       Integer;
  bPScene: PScene;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then
  begin
    bPScene := PScene(bProgram[s]);
    bPScene^.name := InputBox('Rename', 'Please enter new scene name:', bPScene^.name);
    lbScenes.Items[s] := bPScene^.name;
  end;
end;

procedure TformSceneManager.tbDeleteClick(Sender: TObject);
var
  s: Integer;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then
  begin
    PScene(bProgram[s])^.itemIndex.Free;
    PScene(bProgram[s])^.itemData.Free;
    bProgram.Delete(s);
    lbScenes.Items.Delete(s);
    currentScene := nil;
  end;
end;

procedure TformSceneManager.tbPropertiesClick(Sender: TObject);
var
  s: Integer;
  bPScene: PScene;
begin
  s := lbScenes.ItemIndex;
  if s > -1 then
  begin
    bPScene := PScene(bProgram[s]);
    with TformSceneProperties.Create(formSceneManager) do
      try
        udAdvance.Position := bPScene^.advanceTime;
        if ShowModal = mrOK then
          with bPScene^ do
          begin
            advanceTime := udAdvance.Position;
            advanceTimeCountDown := MaxInt;
          end;
      finally
        Free();
      end;
  end;
end;

procedure TformSceneManager.tbUpClick(Sender: TObject);
// added 2004-11-16
var
  s:        Integer;
  bPointer: Pointer;
  bString:  string;
begin
  s := lbScenes.ItemIndex;
  if s > 0 then
  begin
    formLightning.stopTimer();
    try
      // swap scene pointers
      bPointer := bProgram[s - 1];
      bProgram[s - 1] := bProgram[s];
      bProgram[s] := bPointer;

      // swap scene titles in list
      bString := lbScenes.Items[s - 1];
      lbScenes.Items[s - 1] := lbScenes.Items[s];
      lbScenes.Items[s] := bString;

      // currentScene does not need changed,
      // because it is a pointer to a scene rather than an index in the current program

      // always set selection to the scene's new index
      lbScenes.ItemIndex := lbScenes.ItemIndex - 1;
    finally
      formLightning.startTimer();
    end;
  end;
end;

procedure TformSceneManager.tbDownClick(Sender: TObject);
// added 2004-11-16
var
  s:        Integer;
  bPointer: Pointer;
  bString:  string;
begin
  s := lbScenes.ItemIndex;
  if (s > -1) and (s < lbScenes.Count - 1) then
  begin
    formLightning.stopTimer();
    try
      // swap scene pointers
      bPointer := bProgram[s];
      bProgram[s] := bProgram[s + 1];
      bProgram[s + 1] := bPointer;

      // swap scene titles in list
      bString := lbScenes.Items[s];
      lbScenes.Items[s] := lbScenes.Items[s + 1];
      lbScenes.Items[s + 1] := bString;

      // currentScene does not need changed,
      // because it is a pointer to a scene rather than an index in the current program

      // always set selection to the scene's new index
      lbScenes.ItemIndex := lbScenes.ItemIndex + 1;
    finally
      formLightning.startTimer();
    end;
  end;
end;

end.
