unit unitformtrace;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Lightning debug window.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TformTrace = class(TForm)
    memoTrace: TMemo;
    Panel1: TPanel;
    checkBoxRun: TCheckBox;
    procedure checkBoxRunClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    run: Boolean;
    trace: string;
    startTime, endTime: Int64;
  public
    { Public declarations }
    procedure start();
    procedure stop();
    procedure add(text: string);
  end;

var
  formTrace: TformTrace = nil;

implementation

{$R *.dfm}

uses
  common, unitformlightning;

function tickCount(): Int64;
begin
  Result := 0;
  QueryPerformanceCounter(Result);
end;

procedure TformTrace.start();
begin
  if checkBoxRun.Checked then
    trace := '';
  startTime := tickCount();
end;

procedure TformTrace.stop();
begin
  endTime := tickCount;
  formLightning.statusBar.Panels[0].Text := 'processing time = ' + FloatToStr((endTime - startTime) / (tickFreq / 1000)) + ' ms';
  if trace <> memoTrace.Text then
    memoTrace.Text := trace;
end;

procedure TformTrace.add(text: string);
var
  time: string;
begin
  if checkBoxRun.Checked then
  begin
    time := FloatToStr(tickCount() / (tickFreq / 1000));
    while Length(time) < 16 do
      time := time + '0';
    trace := trace + time + ' ms ' + text + #13#10;
  end;
end;

procedure TformTrace.checkBoxRunClick(Sender: TObject);
begin
  if checkBoxRun.Checked then
    trace := '';
end;

procedure TformTrace.FormCreate(Sender: TObject);
begin
  run := False;
  trace := '';
  startTime := 0;
  endTime := 0;
end;

procedure TformTrace.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caNone;
  formLightning.menuItemTraceClick(Sender);
end;

end.
