unit unitmodulecomponentout;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Lightning component support code.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows,
  SysUtils,
  common,
  unitdesktopview;

const
  channels = 32;

//function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
//procedure finish(); cdecl;
function  create(controlHandle: THandle; desktopView: TDesktopView): PLightningModule; cdecl;
procedure destroy(lightningModule: PLightningModule); cdecl;
function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
procedure hide(lightningModule: PLightningModule); cdecl;
procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;

implementation

uses
  unitmodule;

{
function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('', name, nameLength);
end;

procedure finish(); cdecl;
begin
  //
end;
}

function  create(controlHandle: THandle; desktopView: TDesktopView): PLightningModule; cdecl;
var
  i: Integer;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'Component output';
    settings      := False;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := channels;
    outputs       := 0;
    GetMem(inputTooltip, maxLength);
    StrCopy(inputTooltip, '');
    for i := 1 to channels do
      inputTooltip := StrCat(inputTooltip, PChar('Channel ' + IntToStr(i) + ' spectrum|'));
    outputTooltip := '';
    details       := Pointer(desktopView);
    reserved      := nil;
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  FreeMem(lightningModule^.inputTooltip);
  Dispose(lightningModule);
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  desktopView: TDesktopView;
  parentModule: TModule;
begin
  // find corresponding component module
  if lightningModule^.details <> nil then
  begin
    desktopView := TDesktopView(lightningModule^.details);
    if desktopView.Parent <> nil then
    begin
      parentModule := TModule(desktopView.Parent);
      if parentModule.Parent <> nil then
      begin
        // modify the output chain by breaking it and calling action on
        // a corresponding output of the component module
        parentModule.actionOutput(0, input, bridge);
      end;
    end;
  end;
  // else the data originates from component design mode, so ignore the data
  Result := False;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  //
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
begin
  //
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return('', settings, settingsLength);
end;

end.
