unit unitmoduledlop;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Lightning DLOP support code.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows,
  SysUtils,
  Classes,
  Dialogs,
  dlua,
  common;

const
  legacyChannels = 8;

type
  TDLOGetNch = function: Byte; cdecl;
  TDLOGetName = function: PChar; cdecl;
  TDLOOutput = procedure(ch, val : Byte); cdecl;
  TDLOInit = procedure; cdecl;
  TDLODone = procedure; cdecl;
  TDLOAbout = procedure;
  TDetails = class(TObject)
  public
    fileName:    string;
    lib:         HINST;
    dlo_about:   TDLOAbout;
    dlo_output:  TDLOOutput;
    dlo_done:    TDLODone;
    status:      array of Byte;
  end;

//function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
procedure init(); cdecl;
procedure finish(); cdecl;
function  create(controlHandle: THandle; fileName: string): PLightningModule; cdecl;
procedure destroy(lightningModule: PLightningModule); cdecl;
function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
procedure hide(lightningModule: PLightningModule); cdecl;
procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;

implementation

{function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('', name, nameLength);
end;}

procedure init(); cdecl;
begin
  //
end;

procedure finish(); cdecl;
begin
  //
end;

function  create(controlHandle: THandle; fileName: string): PLightningModule; cdecl;
var
  myDetails:   TDetails;
  lib:         HINST;
  i:           Integer;
  dlo_init:    TDLOInit;//FARPROC;
  dlo_getNch:  TDLOGetNch;//FARPROC;
  dlo_getName: TDLOGetName;//FARPROC;
  dlo_about:   TDLOAbout;//FARPROC;
  dlo_output:  TDLOOutput;//FARPROC;
  dlo_done:    TDLODone;//FARPROC;
begin
  @dlo_init    := nil;
  @dlo_getNch  := nil;
  @dlo_getName := nil;
  @dlo_about   := nil;
  @dlo_output  := nil;
  @dlo_done    := nil;
  lib := LoadLibrary(PChar(fileName));
  if lib <> 0 then
  begin
    @dlo_init    := GetProcAddress(lib, 'DLO_init');
    @dlo_getNch  := GetProcAddress(lib, 'DLO_getnch');
    @dlo_getName := GetProcAddress(lib, 'DLO_getname');
    @dlo_about   := GetProcAddress(lib, 'DLO_about');
    @dlo_output  := GetProcAddress(lib, 'DLO_datain');
    @dlo_done    := GetProcAddress(lib, 'DLO_done');
    if (@dlo_getNch <> nil) and (@dlo_getName <> nil) and (@dlo_output <> nil) then
    begin
      if @dlo_init <> nil then
        dlo_init();
    end
    else
    begin
      FreeLibrary(lib);
      lib := INVALID_HANDLE_VALUE;
    end
  end
  else
    lib := INVALID_HANDLE_VALUE;

  New(Result);
  with Result^ do
  begin
    handle := controlHandle;
    if lib <> INVALID_HANDLE_VALUE then
      title := dlo_getName()
    else
      title := 'DLOP not loaded';
    settings  := lib <> INVALID_HANDLE_VALUE;
    width     := -1;
    height    := -1;
    inputTop  := -1;
    infoTop   := -1;
    outputTop := -1;
    if lib <> INVALID_HANDLE_VALUE then
      inputs := dlo_getNch()
    else
      inputs := 0;
    outputs := 0;
    GetMem(inputTooltip, maxLength);
    StrCopy(inputTooltip, '');
    for i := 1 to inputs do
      StrCat(inputTooltip, PChar('DLOP input ' + IntToStr(i) + ' status|'));
    outputTooltip := '';
    myDetails := TDetails.Create();
    details := myDetails;
    reserved := nil;
  end;
  myDetails.lib := lib;
  SetLength(myDetails.status, Result^.inputs);
  if lib <> INVALID_HANDLE_VALUE then
  begin
    @myDetails.dlo_about  := @dlo_about;
    @myDetails.dlo_output := @dlo_output;
    @myDetails.dlo_done   := @dlo_done;
    for i := 0 to Result^.inputs - 1 do // set all channels to 0
    begin
      myDetails.status[i] := 0;
      myDetails.dlo_output(i + 1, 0);
    end;
  end;
  mydetails.fileName := fileName;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    if lightningModule^.inputs > 0 then
    begin
      if @dlo_done <> nil then
        dlo_done();
      FreeLibrary(lib);
    end;
    FreeMem(lightningModule^.inputTooltip);
    Free();
  end;
  Dispose(lightningModule);
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
  bValue:  Byte;
begin
  Result := False;
  if lightningModule^.inputs > 0 then
  begin
    details := TDetails(lightningModule^.details);
    bValue := spectrumAverage(bridge, fftMin, fftMax) div 20;
    if bValue <> details.status[input - 1] then
    begin
      details.status[input - 1] := bValue;
      if bValue = 255 then
        bValue := 1
      else if bValue > 0 then
        Inc(bValue);
      details.dlo_output(input, bValue);
    end;
  end;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
var
  details: TDetails;
begin
  details := TDetails(lightningModule^.details);
  if lightningModule^.inputs > 0 then
    if @details.dlo_about <> nil then
      details.dlo_about();
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
begin
  //
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return('', settings, settingsLength);
end;

end.
