unit unitmodulelua;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Lightning LUA support code.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows,
  SysUtils,
  Classes,
  Dialogs,
  dlua,
  common;

const
  legacyChannels = 8;

type
  TDetails = class(TObject)
  public
    L:         Plua_State;
    init:      Boolean;
    fileName:  string;
  end;

var
  buffer:  Double;

//function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
procedure init(); cdecl;
procedure finish(); cdecl;
function  create(controlHandle: THandle; fileName: string): PLightningModule; cdecl;
procedure destroy(lightningModule: PLightningModule); cdecl;
function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
procedure hide(lightningModule: PLightningModule); cdecl;
procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;

implementation

function  lua_dooutput(L: Plua_State): Integer; cdecl;
begin
  buffer := lua_tonumber(L, lua_gettop(L));
  Result := 0;
end;

{function  init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('', name, nameLength);
end;}

procedure init(); cdecl;
begin
  LoadLua();
  LoadLuaLib();
end;

procedure finish(); cdecl;
begin
  UnLoadLuaLib();
  UnLoadLua();
end;

function  create(controlHandle: THandle; fileName: string): PLightningModule; cdecl;
var
  bString:     string;
  myDetails:   TDetails;
  bStringList: TStringList;
  p:           Integer;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    bString       := ExtractFileName(fileName);
    GetMem(title, Length(bString) + 1);
    StrPCopy(title, bString);
    settings      := False;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := 0;
    outputs       := 0;
    GetMem(inputTooltip, maxLength);
    StrCopy(inputTooltip, '');
    GetMem(outputTooltip, maxLength);
    StrCopy(outputTooltip, '');
    bStringList := TStringList.Create;
    try
      bStringList.LoadFromFile(fileName);
      if bStringList.Count > 0 then
      begin
        // if header is missing, add a header for compatibility with
        // legacy LUA files. 
        if Copy(bStringList[0], 1, 13) <> '-- lightning=' then
        begin
          bStringList.Text :=
            '-- lightning=1.0'#13#10 +
            '-- title=' + title + #13#10 +
            '-- caption=' + title + #13#10 +
            '-- input1=Value 1' + #13#10 +
            '-- input2=Value 2' + #13#10 +
            '-- input3=Value 3' + #13#10 +
            '-- input4=Value 4' + #13#10 +
            '-- input5=Value 5' + #13#10 +
            '-- input6=Value 6' + #13#10 +
            '-- input7=Value 7' + #13#10 +
            '-- input8=Value 8' + #13#10 +
            '-- output1=Processed value 1' + #13#10 +
            '-- output2=Processed value 2' + #13#10 +
            '-- output3=Processed value 3' + #13#10 +
            '-- output4=Processed value 4' + #13#10 +
            '-- output5=Processed value 5' + #13#10 +
            '-- output6=Processed value 6' + #13#10 +
            '-- output7=Processed value 7' + #13#10 +
            '-- output8=Processed value 8' + #13#10 +
            bStringList.Text;
        end;
        bString := bStringList[0];
        bStringList.Delete(0);
        parse(bString, '=');
        if bString = '1.0' then
        begin
          if bStringList.Count > 2 then
          begin
            bStringList.Delete(0);
            bStringList.Delete(0);
            p := Pos('input', bStringList[0]);
            while (p > 0) and (p < Pos('=', bStringList[0])) do
            begin
              bString := bStringList[0];
              bStringList.Delete(0);
              parse(bString, '=');
              inputTooltip := StrCat(inputTooltip, PChar(bString + '|'));
              Inc(inputs);
              p := Pos('input', bStringList[0]);
            end;
            p := Pos('output', bStringList[0]);
            while (p > 0) and (p < Pos('=', bStringList[0])) do
            begin
              bString := bStringList[0];
              bStringList.Delete(0);
              parse(bString, '=');
              outputTooltip := StrCat(outputTooltip, PChar(bString + '|'));
              Inc(outputs);
              p := Pos('output', bStringList[0]);
            end;
          end
          else
            MessageBox(Handle, 'This LUA file is corrupt (2)!.', 'Warning', MB_ICONWARNING);
        end
        else
          MessageBox(Handle, 'This LUA file is too new to be supported by this Lightning version.'#13#10'Please upgrade your Lightning program.', 'Information', MB_ICONINFORMATION);
      end
      else
        MessageBox(Handle, 'This LUA file is corrupt (1)!.', 'Warning', MB_ICONWARNING);
    finally
      bStringList.Free;
    end;
    myDetails     := TDetails.Create();
    details       := myDetails;
    reserved      := nil;
  end;
  with myDetails do
  begin
    L := lua_open();
    //if not(lua_baselibopen(L)) then ShowMessage('err');
    lua_register(L, 'dooutput', lua_dooutput);
    init := False;
  end;
  mydetails.fileName := fileName;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule^.details) do
  begin
    lua_close(L);
    Free();
  end;
  FreeMem(lightningModule^.outputTooltip);
  FreeMem(lightningModule^.inputTooltip);
  FreeMem(lightningModule^.title);
  Dispose(lightningModule);
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
label
  next;
var
  details:   TDetails;
  connected,
  accept:    Boolean; // accept is True if the input is accepted; Result decides whether the script's bridge data will be output 
  lastConnectedInput{, {i, }{channel}: Integer;
  {fileName, }bString: string;
begin
  Result := False;
  if (lightningModule^.inputs + lightningModule^.outputs) > 0 then
  begin
    {if lightningModule^.outputs > lightningModule^.inputs then
    begin
      Result := (input = output);
      if not(Result) and (output > lightningModule^.inputs) then
      begin
        bString := inputConnected;
        lastConnectedInput := 0;
        while bString <> '' do
          if parse(bString, '|') <> '' then
            Inc(lastConnectedInput);
        Result := input = lastConnectedInput;
      end;
    end
    else}
    connected := True;
    accept := (input = output) or // straight forward; channel = input = output
              ((input > lightningModule^.outputs) and (output <= 1)); // allow input; ignore output; channel = input
    Result := accept
              and
              not(input > lightningModule^.outputs);
    if not({Result}accept) then
    begin
      connected := False;

      bString := inputConnected;
      lastConnectedInput := 0;
      while bString <> '' do
        if parse(bString, '|') <> '' then
          Inc(lastConnectedInput);
      if input = lastConnectedInput then // last chance to process so far unprocessed outputs
        if output > lightningModule^.inputs then // outputs beyond the input range are always unprocessed so far
        begin
          accept := True;
          Result := True; // channel = output
        end;
        {else // other outputs aren't already processed so far only if the corresponding input is unconnected
        begin
          bString := inputConnected;
          i := output;
          while i > 1 do
          begin
            parse(bString, '|');
            Dec(i);
          end;
          Result := parse(bString, '|') = ''; // channel = output
        end;}
      // else // ignore input; ignore output
    end;
    details := TDetails(lightningModule^.details);
    //fileName := ExtractFilePath(ParamStr(0)) + Trim(details.fileName);
    if (Trim(details.fileName) <> '') and FileExists(details.fileName) then
    begin
      if not(details.init) then
      begin
        lua_dostring(details.L, 'bridge = 0');
        lua_dostring(details.L, 'channel = 0');
        if Trim(details.fileName) <> '' then
          lua_dofile(details.L, PChar(details.fileName));

        details.init := True;
      end;
      if {Result}accept then
      begin
        if connected then
        begin
          lua_dostring(details.L, 'connected = 1');
          lua_dostring(details.L, PChar('bridge = ' + IntToStr(spectrumAverage(bridge, fftMin, fftMax))));
        end
        else
        begin
          lua_dostring(details.L, 'connected = 0');
          lua_dostring(details.L, 'bridge = 0');
        end;
        if input > lightningModule^.outputs then
        begin
          lua_dostring(details.L, PChar('channel = ' + IntToStr(input{output})));
          //channel := input{output};
        end
        else
        begin
          lua_dostring(details.L, PChar('channel = ' + IntToStr(output)));
          //channel := output;
        end;
        if Trim(details.fileName) <> '' then
          lua_dofile(details.L, PChar(details.fileName));
        if Result then
        begin
          lua_dostring(details.L, 'dooutput(bridge)');
          if buffer > 5119 then
            buffer := 5119
          else if buffer < 0 then
            buffer := 0;
          createSpectrum(bridge, Trunc(buffer));
        end;
        {ShowMessage(
          'Channel: ' + IntToStr(channel) + #13#10 +
          'Input: ' + IntToStr(input) + #13#10 +
          'Output: ' + IntToStr(output) + #13#10 +
          'Buffer: ' + FloatToStr(buffer) + #13#10 +
          'Accept: ' + IntToStr(Ord(accept)) + #13#10 +
          'Result: ' + IntToStr(Ord(Result)) + #13#10
        );}
      end;
    end;
  end;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  //
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  //
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
begin
  //
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
begin
  Result := return('', settings, settingsLength);
end;

end.
